using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using ApiGateway;
using Microsoft.AspNetCore.Builder;
using Microsoft.AspNetCore.Hosting;
using Microsoft.AspNetCore.Http;
using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.DependencyInjection;
using Microsoft.Extensions.Hosting;
using Nacos.AspNetCore.V2;
using Ocelot.Configuration;
using Ocelot.DependencyInjection;
using Ocelot.LoadBalancer.LoadBalancers;
using Ocelot.Middleware;
using Ocelot.Provider.Nacos;
using Ocelot.ServiceDiscovery;
using Ocelot.ServiceDiscovery.Providers;

namespace ApiGateway
{
    public class Startup
    {
        public Startup(IConfiguration configuration)
        {
            Configuration = configuration;
        }

        public IConfiguration Configuration { get; }
        Func<IServiceProvider, DownstreamRoute, IServiceDiscoveryProvider, WeightRandom> loadBalancerFactoryFunc = (serviceProvider, Route, serviceDiscoveryProvider) => new WeightRandom(serviceDiscoveryProvider.Get);
        public void ConfigureServices(IServiceCollection services)
        {
            //services.AddOcelot().AddNacosDiscovery();
            services.AddOcelot(Configuration)
                .AddNacosDiscovery()
                .AddCustomLoadBalancer(loadBalancerFactoryFunc);
            services.AddNacosAspNet(Configuration, "nacos");
        }

        // This method gets called by the runtime. Use this method to configure the HTTP request pipeline.
        public void Configure(IApplicationBuilder app, IWebHostEnvironment env)
        {

            app.UseOcelot().Wait();
        }
    }
}
