﻿using Microsoft.AspNetCore.Http;
using Microsoft.Extensions.Configuration;
using Ocelot.LoadBalancer.LoadBalancers;
using Ocelot.Responses;
using Ocelot.Values;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;

namespace ApiGateway
{
    /// <summary>
    /// 自定 义负载均衡策略
    /// </summary>
    public class WeightRandom : ILoadBalancer
    {
        private readonly IConfiguration _configuration;
        private readonly object _lock = new object();
        private readonly Func<Task<List<Service>>> _services;
        private int _last;

        public WeightRandom(Func<Task<List<Service>>> services)
        {
            _services = services;
        }

        public async Task<Response<ServiceHostAndPort>> Lease(HttpContext httpContext)
        {
            //var config = _configuration["appName"];
            var services = await
                _services();

            if (services == null ||
                services.Count == 0)
            {
                return null;
            }

            lock (_lock)
            {
                var healthyServices = services;

                if (services.Count == 0)
                {
                    return null;
                }

                var totalWeight = services.Sum(s => int.Parse(s.Version));
                var randomWeight = new Random().Next(totalWeight);

                var selectedService = services.First();
                foreach (var serviceItem in services)
                {
                    if (randomWeight < int.Parse(serviceItem.Version))
                    {
                        selectedService = serviceItem;
                        break;
                    }

                    randomWeight -= int.Parse(serviceItem.Version);
                }

                return new OkResponse<ServiceHostAndPort>(new ServiceHostAndPort(selectedService.HostAndPort.DownstreamHost, selectedService.HostAndPort.DownstreamPort));
            }
        }

        public void Release(ServiceHostAndPort hostAndPort)
        {
        }
    }
}